@@ -1,3 +1,5 @@
+#ifndef ERROR_H
+#define ERROR_H
 #include <stdarg.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -5,3 +7,5 @@
 void fatal(char *fmt, ...);
 void error(char *fmt, ...);
 void pfatal(char *err);
+#endif /* ERROR_H */
+
