@@ -9,10 +9,6 @@
 #include "config.h"
 #endif
 
-#ifndef HAVE_INLINE
-#define __inline__
-#endif
-
 #ifdef STDC_HEADERS
 #include <stdlib.h>
 #else
@@ -30,6 +26,10 @@ void *realloc();
 #include <strings.h>
 #endif
 
+#ifdef HAVE_BSTRING_H
+#include <bstring.h>
+#endif
+
 #ifndef HAVE_BZERO
 #define bzero(s, n) memset((s), 0, (n))
 #endif
@@ -72,11 +72,23 @@ void *realloc();
 #include <netinet/in.h>
 #include <errno.h>
 #include <netdb.h>
-#include <time.h>
+
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #include <fcntl.h>
 #include <signal.h> 
 #include <signal.h>
 #include <stdarg.h>
+#include <pwd.h>
 #ifndef NETINET_IN_SYSTEM_H  /* why the HELL does OpenBSD not do this? */
 #include <netinet/in_systm.h> /* defines n_long needed for netinet/ip.h */
 #define NETINET_IN_SYSTEM_H
@@ -88,7 +100,7 @@ void *realloc();
 #include <netinet/ip_icmp.h> 
 #include <arpa/inet.h>
 #include <math.h>
-#include <sys/time.h> 
+#include <assert.h>
 #ifndef __FAVOR_BSD
 #define __FAVOR_BSD
 #endif
@@ -109,17 +121,15 @@ void *realloc();
 #endif /* NETINET_IF_ETHER_H */
 #endif /* HAVE_NETINET_IF_ETHER_H */
 
-#include "tcpip.h"
-#include "error.h"
-#include "utils.h"
-
 /*******  DEFINES  ************/
 
 /* User configurable #defines: */
 /* #define to zero if you don't want to	ignore hosts of the form 
    xxx.xxx.xxx.{0,255} (usually network and broadcast addresses) */
 #define IGNORE_ZERO_AND_255_HOSTS 0
-#define VERSION "1.51"
+#ifndef VERSION
+#define VERSION "1.60-Beta"
+#endif
 #ifndef DEBUGGING
 #define DEBUGGING 0
 #endif
@@ -130,10 +140,12 @@ void *realloc();
    high value (avoids dynamic memmory allocation */
 #define MAX_SOCKETS_ALLOWED 1025
 /* How many hosts do we ping in parallel to see if they are up? */
-#define LOOKAHEAD 52
+#define LOOKAHEAD 25
 /* If reads of a UDP port keep returning EAGAIN (errno 13), do we want to 
    count the port as valid? */
 #define RISKY_UDP_SCAN 0
+/* How many syn packets do we send to TCP sequence a host? */
+#define NUM_SEQ_SAMPLES 6
  /* This ideally should be a port that isn't in use for any protocol on our machine or on the target */ 
 #define MAGIC_PORT 49724
 /* How many udp sends without a ICMP port unreachable error does it take before we consider the port open? */
@@ -149,6 +161,9 @@ void *realloc();
 #define MAX_TIMEOUTS MAX_SOCKETS   /* How many timed out connection attempts 
 				      in a row before we decide the host is 
 				      dead? */
+#define MAX_DECOYS 128 /* How many decoys are allowed? */
+
+
 
 /* DO NOT change stuff after this point */
 #define UC(b)   (((int)b)&0xff)
@@ -162,95 +177,59 @@ void *realloc();
 #define HOST_FIREWALLED 4 
 #define HOST_BROADCAST 8 /* use the wierd_responses member of hoststruct instead */
 /* struct port stuff */
-#define PORT_CLOSED 0;
-#define PORT_OPEN 1;
-#define CONF_NONE 0;
-#define CONF_LOW 1;
-#define CONF_HIGH 2;
+#define PORT_UNKNOWN 0
+#define PORT_CLOSED 1
+#define PORT_OPEN 2
+#define PORT_FIREWALLED 4
+#define PORT_TESTING 8
+#define PORT_FRESH 16
+#define PORT_UNFIREWALLED 32
+ 
+#define CONF_NONE 0
+#define CONF_LOW 1
+#define CONF_HIGH 2
+
+#define PINGTYPE_UNKNOWN 0
+#define PINGTYPE_NONE 1
+#define PINGTYPE_ICMP 2
+#define PINGTYPE_TCP  4
+#define PINGTYPE_TCP_USE_ACK 8
+#define PINGTYPE_TCP_USE_SYN 16
+#define PINGTYPE_RAWTCP 32
+#define PINGTYPE_CONNECTTCP 64
+
+#define SEQ_UNKNOWN 0
+#define SEQ_64K 1
+#define SEQ_TD 2
+#define SEQ_RI 4
+#define SEQ_TR 8
+#define SEQ_i800 16
+#define SEQ_CONSTANT 32
 
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 64
 #endif
 
-#ifdef FREEBSD
+#ifndef BSDFIX
+#if FREEBSD || BSDI || NETBSD
 #define BSDFIX(x) x
 #define BSDUFIX(x) x
 #else
 #define BSDFIX(x) htons(x)
 #define BSDUFIX(x) ntohs(x)
 #endif
+#endif /* BSDFIX */
+/********************** LOCAL INCLUDES *****************************/
+
+#include "tcpip.h"
+#include "global_structures.h"
+#include "error.h"
+#include "utils.h"
+#include "services.h"
 
 /***********************STRUCTURES**********************************/
 
-typedef struct port {
-  unsigned short portno;
-  unsigned char proto;
-  char *owner;
-  int state; 
-  int confidence; /* How sure are we about the state? */
-  struct port *next;
-} port;
-
-struct ftpinfo {
-  char user[64];
-  char pass[256]; /* methinks you're paranoid if you need this much space */
-  char server_name[MAXHOSTNAMELEN + 1];
-  struct in_addr server;
-  unsigned short port;
-  int sd; /* socket descriptor */
-};
-
-struct targets {
-  /* These 4 are used for the '/mask' style of specifying target net*/
-  unsigned int netmask;
-  unsigned int maskformat;
-  struct in_addr start;
-  struct in_addr currentaddr;
-  struct in_addr end;
-  /* These two are for the '138.[1-7,16,91-95,200-].12.1 style */
-  unsigned char addresses[4][256];
-  unsigned int current[4];
-  unsigned char last[4];
-};
-
-struct hoststruct {
-  struct in_addr host;
-  struct in_addr source_ip;
-  char *name;
-  struct port *ports;
-  /*
-  unsigned int up;
-  unsigned int down; */
-  int wierd_responses; /* echo responses from other addresses, Ie a network broadcast address */
-  unsigned int flags; /* HOST_UP, HOST_DOWN, HOST_FIREWALLED, HOST_BROADCAST (instead of HOST_BROADCAST use wierd_responses */
-  unsigned long rtt; /* microseconds */
-  char device[64]; /* The device we transmit on */
-};
-
-struct ops /* someone took struct options, <grrr> */ {
-  int debugging;
-  int verbose;
-  int spoofsource; /* -S used */
-  int number_of_ports;
-  int max_sockets;
-  int isr00t;
-  int identscan;
-  int dontping;
-  int allowall;
-  int wait;
-  int ptime;
-  int numports;
-  int fragscan;
-  int synscan;
-  int finscan;
-  int udpscan;
-  int lamerscan;
-  int noresolve;
-  int force; /* force nmap to continue on even when the outcome seems somewhat certain */
-  FILE *logfd; /* Output log file descriptor */
-};
-  
-typedef port *portlist;
+/* Moved to global_structures.h */
 
 /***********************PROTOTYPES**********************************/
 
@@ -258,11 +237,8 @@ typedef port *portlist;
 void printusage(char *name);
 
 /* our scanning functions */
-portlist tcp_scan(struct hoststruct *target, unsigned short *portarray,                  int timeout);
-portlist syn_scan(struct hoststruct *target, unsigned short *portarray);
-portlist fin_scan(struct hoststruct *target, unsigned short *portarray);
-portlist udp_scan(struct hoststruct *target, unsigned short *portarray);
-portlist lamer_udp_scan(struct hoststruct *target,unsigned short *portarray);
+portlist super_scan(struct hoststruct *target, unsigned short *portarray, stype scantype);
+portlist pos_scan(struct hoststruct *target, unsigned short *portarray, stype scantype);
 portlist bounce_scan(struct hoststruct *target, unsigned short *portarray,
 		     struct ftpinfo *ftp);
 
@@ -273,13 +249,26 @@ int getidentinfoz(struct in_addr target, int localport, int remoteport,
 		  char *owner);
 int parse_bounce(struct ftpinfo *ftp, char *url);
 int ftp_anon_connect(struct ftpinfo *ftp);
-
+/* Does the appropriate stuff when the port we are looking at is found
+   to be open trynum is the try number that was successful */
+void posportupdate(struct hoststruct *target, struct portinfo *current, 
+		   int trynum, struct portinfo *scan,
+		   struct scanstats *ss ,stype scantype, int newstate,
+		   struct portinfolist *pil, struct connectsockinfo *csi);
+void get_syn_results(struct hoststruct *target, struct portinfo *scan,
+		     struct scanstats *ss, struct portinfolist *pil, 
+		     int *portlookup, pcap_t *pd, unsigned long *sequences);
+int get_connect_results(struct hoststruct *target, struct portinfo *scan, 
+			 struct scanstats *ss, struct portinfolist *pil, 
+			 int *portlookup, unsigned long *sequences, 
+			 struct connectsockinfo *csi);
+inline void adjust_timeouts(struct timeval sent, struct timeout_info *to);
 /* port manipulators */
 unsigned short *getpts(char *expr); /* someone stole the name getports()! */
-unsigned short *getfastports(int tcpscan, int udpscan);
 int addport(portlist *ports, unsigned short portno, unsigned short protocol,
-	    char *owner);
+	    char *owner, int state);
 int deleteport(portlist *ports, unsigned short portno, unsigned short protocol);
+struct port *lookupport(portlist ports, unsigned short portno, unsigned short protocol);
 void printandfreeports(portlist ports);
 int shortfry(unsigned short *ports);
 
@@ -291,29 +280,45 @@ void broadcast_socket(int sd);
 int recvtime(int sd, char *buf, int len, int seconds);
 void max_rcvbuf(int sd);
 int max_sd();
+
 /* RAW packet building/dissasembling stuff */
 int isup(struct in_addr target);
 int send_small_fragz(int sd, struct in_addr *source, struct in_addr *victim,
-		     int sport, int dport, int flags);
+		     unsigned long seq, int sport, int dport, int flags);
 int listen_icmp(int icmpsock, unsigned short outports[],
 		unsigned short numtries[], int *num_out,
 		struct in_addr target, portlist *ports);
 void massping(struct hoststruct *hostbatch, int numhosts, int pingtimeout);
+
 /* general helper functions */
 void hdump(unsigned char *packet, int len);
 void *safe_malloc(int size);
+char *grab_next_host_spec(FILE *inputfd, int argc, char **fakeargv);
 int parse_targets(struct targets *targets, char *h);
 struct hoststruct *nexthost(char *hostexp, int lookahead, int pingtimeout);
 void options_init();
 void nmap_log(char *fmt, ...);
+void nmap_machine_log(char *fmt, ...);
+char *statenum2str(int state);
 void sigdie(int signo);
+char *seqreport(struct seq_info *seq);
+char *seqclass2ascii(int clas);
+void invertfirewalled(portlist *pl, unsigned short *ports);
+int nmap_fetchfile(char *filename_returned, int bufferlen, char *file);
+int fileexistsandisreadable(char *pathname);
 /* From glibc 2.0.6 because Solaris doesn't seem to have this function */
 #ifndef HAVE_INET_ATON
 int inet_aton(register const char *, struct in_addr *);
 #endif
+#ifndef HAVE_SNPRINTF
+int snprintf ( char *str, size_t n, const char *format, ... );
+#endif
 #endif /* NMAP_H */
 
 
 
 
 
+
+
+
