@@ -5,10 +5,6 @@
 #include "config.h"
 #endif
 
-#ifndef HAVE_INLINE
-#define __inline__
-#endif
-
 #ifdef STDC_HEADERS
 #include <stdlib.h>
 #else
@@ -26,6 +22,10 @@ void *realloc();
 #include <strings.h>
 #endif
 
+#ifdef HAVE_BSTRING_H
+#include <bstring.h>
+#endif
+
 #ifndef HAVE_BZERO
 #define bzero(s, n) memset((s), 0, (n))
 #endif
@@ -76,6 +76,10 @@ void *realloc();
 #include <netinet/tcp.h>          /*#include <netinet/ip_tcp.h>*/
 #define NETINET_TCP_H
 #endif
+#ifndef NETINET_UDP_H
+#include <netinet/udp.h>
+#define NETINET_UDP_H
+#endif
 #include <unistd.h>
 #include <fcntl.h>
 #ifndef NET_IF_H  /* why the HELL does OpenBSD not do this? */
@@ -88,8 +92,18 @@ void *realloc();
 #define NETINET_IF_ETHER_H
 #endif /* NETINET_IF_ETHER_H */
 #endif /* HAVE_NETINET_IF_ETHER_H */
-#include <sys/time.h>
-/*#include <time.h>  does your system need this instead? */
+
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #include <sys/ioctl.h>
 #include <pcap.h>
 #include <setjmp.h>
@@ -110,13 +124,27 @@ void *realloc();
 #define TCPIP_DEBUGGING 0
 #endif
 
-#ifdef FREEBSD
+#ifndef BSDFIX
+#if FREEBSD || BSDI || NETBSD
 #define BSDFIX(x) x
 #define BSDUFIX(x) x
 #else
 #define BSDFIX(x) htons(x)
 #define BSDUFIX(x) ntohs(x)
 #endif
+#endif
+
+/* Bogus TCP flag!!!  Krad! */
+#define TH_BOGUS 64
+#define TH_BOG 64
+
+struct interface_info {
+    char name[64];
+    struct in_addr addr;
+};
+
+
+
 
 #ifndef HAVE_STRUCT_IP
 #define HAVE_STRUCT_IP
@@ -151,6 +179,17 @@ struct ip
 
 #endif /* HAVE_STRUCT_IP */
 
+#ifdef LINUX
+typedef struct udphdr_bsd {
+         unsigned short uh_sport;           /* source port */
+         unsigned short uh_dport;           /* destination port */
+         unsigned short uh_ulen;            /* udp length */
+         unsigned short uh_sum;             /* udp checksum */
+} udphdr_bsd;
+#else
+ typedef struct udphdr udphdr_bsd;
+#endif 
+
 
 #ifndef HAVE_STRUCT_ICMP
 #define HAVE_STRUCT_ICMP
@@ -243,30 +282,49 @@ struct icmp
    result in ip .  returns 0 if hostname cannot
    be resolved */
 int resolve(char *hostname, struct in_addr *ip);
+/* An awesome function to determine what interface a packet to a given
+   destination should be routed through.  It returns NULL if no appropriate
+   interface is found, oterwise it returns the device name and fills in the
+   source parameter */
+char *routethrough(struct in_addr *dest, struct in_addr *source);
 unsigned short in_cksum(unsigned short *ptr,int nbytes);
-int send_tcp_raw( int sd, struct in_addr *source,
-                  struct in_addr *victim, unsigned short sport,
-                  unsigned short dport, unsigned long seq,
-                  unsigned long ack, unsigned char flags,
-                  unsigned short window, char *data,
-                  unsigned short datalen);
-/* A simple program I wrote to help in debugging, shows the important fields
+int send_tcp_raw( int sd, struct in_addr *source, 
+		  struct in_addr *victim, unsigned short sport, 
+		  unsigned short dport, unsigned long seq,
+		  unsigned long ack, unsigned char flags,
+		  unsigned short window, char *options, int optlen,
+		  char *data, unsigned short datalen);
+int send_udp_raw( int sd, struct in_addr *source, 
+		  struct in_addr *victim, unsigned short sport, 
+		  unsigned short dport, char *data, unsigned short datalen);
+int send_ip_raw( int sd, struct in_addr *source, 
+		  struct in_addr *victim, unsigned char proto,
+		  char *data, unsigned short datalen);
+/* A simple function I wrote to help in debugging, shows the important fields
    of a TCP packet*/
 int readtcppacket(char *packet, int readdata);
+int readudppacket(char *packet, int readdata);
 /* Convert an IP address to the device (IE ppp0 eth0) using that address */
 int ipaddr2devname( char *dev, struct in_addr *addr );
-void sethdrinclude(int sd);
+/* And vice versa */
+int devname2ipaddr(char *dev, struct in_addr *addr);
+/* Where the above 2 functions get their info */
+struct interface_info *getinterfaces(int *howmany);
+inline void sethdrinclude(int sd);
 int getsourceip(struct in_addr *src, struct in_addr *dst);
 /* Get the source IP and interface name that a packet
    to dst should be sent to.  Interface name is dynamically
    assigned and thus should be freed */
 char *getsourceif(struct in_addr *src, struct in_addr *dst);
+int islocalhost(struct in_addr *addr);
 int unblock_socket(int sd);
+inline int Sendto(char *functionname, int sd, char *packet, int len, 
+	   unsigned int flags, struct sockaddr *to, int tolen);
 /* Standard swiped internet checksum routine */
 unsigned short in_cksum(unsigned short *ptr,int nbytes);
 /* Hex dump */
 int get_link_offset(char *device);
-char *readip_pcap(pcap_t *pd, unsigned int *len);
+char *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec);
 char *readip_pcap_timed(pcap_t *pd, unsigned int *len, unsigned long timeout /*seconds
  */);
 #ifndef HAVE_INET_ATON
